package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.AcceptanceRecordForm;
import digital.laboratory.platform.reagent.service.AcceptanceRecordFormService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (验收记录表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (验收记录表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/acceptance_record_form" )
@Api(value = "acceptance_record_form", tags = "(验收记录表)管理")
public class AcceptanceRecordFormController {

    private final  AcceptanceRecordFormService acceptanceRecordFormService;

	/**
	 * 通过id查询(验收记录表)
	 * @param acceptanceRecordFormId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{acceptanceRecordFormId}" )
	@PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_get')" )
	public R<AcceptanceRecordForm> getById(@PathVariable("acceptanceRecordFormId" ) String acceptanceRecordFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        AcceptanceRecordForm acceptanceRecordForm = acceptanceRecordFormService.getById(acceptanceRecordFormId);
		return R.ok(acceptanceRecordForm);
		//return R.ok(acceptanceRecordFormService.getById(acceptanceRecordFormId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param acceptanceRecordForm (验收记录表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_get')" )
    public R<IPage<AcceptanceRecordForm>> getAcceptanceRecordFormPage(Page<AcceptanceRecordForm> page, AcceptanceRecordForm acceptanceRecordForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<AcceptanceRecordForm> acceptanceRecordFormSList = acceptanceRecordFormService.page(page, Wrappers.<AcceptanceRecordForm>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(acceptanceRecordFormSList);
//		return R.ok(acceptanceRecordFormService.page(page, Wrappers.query(acceptanceRecordForm)));
    }


    /**
     * 新增(验收记录表)
     * @param acceptanceRecordForm (验收记录表)
     * @return R
     */
    @ApiOperation(value = "新增(验收记录表)", notes = "新增(验收记录表)")
    @SysLog("新增(验收记录表)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_add')" )
    public R<AcceptanceRecordForm> postAddObject(@RequestBody AcceptanceRecordForm acceptanceRecordForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        acceptanceRecordForm.setAcceptanceRecordFormId(IdWorker.get32UUID().toUpperCase());
		if (acceptanceRecordFormService.save(acceptanceRecordForm)) {
			return R.ok(acceptanceRecordForm, "对象创建成功");
		}
		else {
			return R.failed(acceptanceRecordForm, "对象创建失败");
		}
    }

    /**
     * 修改(验收记录表)
     * @param acceptanceRecordForm (验收记录表)
     * @return R
     */
    @ApiOperation(value = "修改(验收记录表)", notes = "修改(验收记录表)")
    @SysLog("修改(验收记录表)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_edit')" )
    public R<AcceptanceRecordForm> putUpdateById(@RequestBody AcceptanceRecordForm acceptanceRecordForm, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (acceptanceRecordFormService.updateById(acceptanceRecordForm)) {
			return R.ok(acceptanceRecordForm, "保存对象成功");
		}
		else {
			return R.failed(acceptanceRecordForm, "保存对象失败");
		}
    }

    /**
     * 通过id删除(验收记录表)
     * @param acceptanceRecordFormId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(验收记录表)", notes = "通过id删除(验收记录表)")
    @SysLog("通过id删除(验收记录表)" )
    @DeleteMapping("/{acceptanceRecordFormId}" )
    @PreAuthorize("@pms.hasPermission('reagent_acceptance_record_form_del')" )
    public R<AcceptanceRecordForm> deleteById(@PathVariable String acceptanceRecordFormId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        AcceptanceRecordForm oldAcceptanceRecordForm = acceptanceRecordFormService.getById(acceptanceRecordFormId);

		if (acceptanceRecordFormService.removeById(acceptanceRecordFormId)) {
			return R.ok(oldAcceptanceRecordForm, "对象删除成功");
		}
		else {
			return R.failed(oldAcceptanceRecordForm, "对象删除失败");
		}

    }

}
