package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.AfterSaleSituation;
import digital.laboratory.platform.reagent.service.AfterSaleSituationService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * 服务商/供应商响应、资质和售后情况
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 服务商/供应商响应、资质和售后情况 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/after_sale_situation" )
@Api(value = "after_sale_situation", tags = "服务商/供应商响应、资质和售后情况管理")
public class AfterSaleSituationController {

    private final  AfterSaleSituationService afterSaleSituationService;

	/**
	 * 通过id查询服务商/供应商响应、资质和售后情况
	 * @param afterSaleSituationId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{afterSaleSituationId}" )
	@PreAuthorize("@pms.hasPermission('reagent_after_sale_situation_get')" )
	public R<AfterSaleSituation> getById(@PathVariable("afterSaleSituationId" ) String afterSaleSituationId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        AfterSaleSituation afterSaleSituation = afterSaleSituationService.getById(afterSaleSituationId);
		return R.ok(afterSaleSituation);
		//return R.ok(afterSaleSituationService.getById(afterSaleSituationId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param afterSaleSituation 服务商/供应商响应、资质和售后情况
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_after_sale_situation_get')" )
    public R<IPage<AfterSaleSituation>> getAfterSaleSituationPage(Page<AfterSaleSituation> page, AfterSaleSituation afterSaleSituation, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<AfterSaleSituation> afterSaleSituationSList = afterSaleSituationService.page(page, Wrappers.<AfterSaleSituation>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(afterSaleSituationSList);
//		return R.ok(afterSaleSituationService.page(page, Wrappers.query(afterSaleSituation)));
    }


    /**
     * 新增服务商/供应商响应、资质和售后情况
     * @param afterSaleSituation 服务商/供应商响应、资质和售后情况
     * @return R
     */
    @ApiOperation(value = "新增服务商/供应商响应、资质和售后情况", notes = "新增服务商/供应商响应、资质和售后情况")
    @SysLog("新增服务商/供应商响应、资质和售后情况" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_after_sale_situation_add')" )
    public R<AfterSaleSituation> postAddObject(@RequestBody AfterSaleSituation afterSaleSituation, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        afterSaleSituation.setAfterSaleSituationId(IdWorker.get32UUID().toUpperCase());
		if (afterSaleSituationService.save(afterSaleSituation)) {
			return R.ok(afterSaleSituation, "对象创建成功");
		}
		else {
			return R.failed(afterSaleSituation, "对象创建失败");
		}
    }

    /**
     * 修改服务商/供应商响应、资质和售后情况
     * @param afterSaleSituation 服务商/供应商响应、资质和售后情况
     * @return R
     */
    @ApiOperation(value = "修改服务商/供应商响应、资质和售后情况", notes = "修改服务商/供应商响应、资质和售后情况")
    @SysLog("修改服务商/供应商响应、资质和售后情况" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_after_sale_situation_edit')" )
    public R<AfterSaleSituation> putUpdateById(@RequestBody AfterSaleSituation afterSaleSituation, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (afterSaleSituationService.updateById(afterSaleSituation)) {
			return R.ok(afterSaleSituation, "保存对象成功");
		}
		else {
			return R.failed(afterSaleSituation, "保存对象失败");
		}
    }

    /**
     * 通过id删除服务商/供应商响应、资质和售后情况
     * @param afterSaleSituationId id
     * @return R
     */
    @ApiOperation(value = "通过id删除服务商/供应商响应、资质和售后情况", notes = "通过id删除服务商/供应商响应、资质和售后情况")
    @SysLog("通过id删除服务商/供应商响应、资质和售后情况" )
    @DeleteMapping("/{afterSaleSituationId}" )
    @PreAuthorize("@pms.hasPermission('reagent_after_sale_situation_del')" )
    public R<AfterSaleSituation> deleteById(@PathVariable String afterSaleSituationId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        AfterSaleSituation oldAfterSaleSituation = afterSaleSituationService.getById(afterSaleSituationId);

		if (afterSaleSituationService.removeById(afterSaleSituationId)) {
			return R.ok(oldAfterSaleSituation, "对象删除成功");
		}
		else {
			return R.failed(oldAfterSaleSituation, "对象删除失败");
		}

    }

}
