package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.ApplicationForUse;
import digital.laboratory.platform.reagent.service.ApplicationForUseService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (试剂耗材领用申请表)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (试剂耗材领用申请表) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/application_for_use" )
@Api(value = "application_for_use", tags = "(试剂耗材领用申请表)管理")
public class ApplicationForUseController {

    private final  ApplicationForUseService applicationForUseService;

	/**
	 * 通过id查询(试剂耗材领用申请表)
	 * @param applicationForUseId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{applicationForUseId}" )
	@PreAuthorize("@pms.hasPermission('reagent_application_for_use_get')" )
	public R<ApplicationForUse> getById(@PathVariable("applicationForUseId" ) String applicationForUseId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ApplicationForUse applicationForUse = applicationForUseService.getById(applicationForUseId);
		return R.ok(applicationForUse);
		//return R.ok(applicationForUseService.getById(applicationForUseId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param applicationForUse (试剂耗材领用申请表)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_get')" )
    public R<IPage<ApplicationForUse>> getApplicationForUsePage(Page<ApplicationForUse> page, ApplicationForUse applicationForUse, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<ApplicationForUse> applicationForUseSList = applicationForUseService.page(page, Wrappers.<ApplicationForUse>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(applicationForUseSList);
//		return R.ok(applicationForUseService.page(page, Wrappers.query(applicationForUse)));
    }


    /**
     * 新增(试剂耗材领用申请表)
     * @param applicationForUse (试剂耗材领用申请表)
     * @return R
     */
    @ApiOperation(value = "新增(试剂耗材领用申请表)", notes = "新增(试剂耗材领用申请表)")
    @SysLog("新增(试剂耗材领用申请表)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_add')" )
    public R<ApplicationForUse> postAddObject(@RequestBody ApplicationForUse applicationForUse, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        applicationForUse.setApplicationForUseId(IdWorker.get32UUID().toUpperCase());
		if (applicationForUseService.save(applicationForUse)) {
			return R.ok(applicationForUse, "对象创建成功");
		}
		else {
			return R.failed(applicationForUse, "对象创建失败");
		}
    }

    /**
     * 修改(试剂耗材领用申请表)
     * @param applicationForUse (试剂耗材领用申请表)
     * @return R
     */
    @ApiOperation(value = "修改(试剂耗材领用申请表)", notes = "修改(试剂耗材领用申请表)")
    @SysLog("修改(试剂耗材领用申请表)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_edit')" )
    public R<ApplicationForUse> putUpdateById(@RequestBody ApplicationForUse applicationForUse, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (applicationForUseService.updateById(applicationForUse)) {
			return R.ok(applicationForUse, "保存对象成功");
		}
		else {
			return R.failed(applicationForUse, "保存对象失败");
		}
    }

    /**
     * 通过id删除(试剂耗材领用申请表)
     * @param applicationForUseId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(试剂耗材领用申请表)", notes = "通过id删除(试剂耗材领用申请表)")
    @SysLog("通过id删除(试剂耗材领用申请表)" )
    @DeleteMapping("/{applicationForUseId}" )
    @PreAuthorize("@pms.hasPermission('reagent_application_for_use_del')" )
    public R<ApplicationForUse> deleteById(@PathVariable String applicationForUseId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        ApplicationForUse oldApplicationForUse = applicationForUseService.getById(applicationForUseId);

		if (applicationForUseService.removeById(applicationForUseId)) {
			return R.ok(oldApplicationForUse, "对象删除成功");
		}
		else {
			return R.failed(oldApplicationForUse, "对象删除失败");
		}

    }

}
