package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.EvaluationResult;
import digital.laboratory.platform.reagent.service.EvaluationResultService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * (服务商/供应商评价结果)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (服务商/供应商评价结果) 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/evaluation_result" )
@Api(value = "evaluation_result", tags = "(服务商/供应商评价结果)管理")
public class EvaluationResultController {

    private final  EvaluationResultService evaluationResultService;

	/**
	 * 通过id查询(服务商/供应商评价结果)
	 * @param evaluationResultId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{evaluationResultId}" )
	@PreAuthorize("@pms.hasPermission('reagent_evaluation_result_get')" )
	public R<EvaluationResult> getById(@PathVariable("evaluationResultId" ) String evaluationResultId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        EvaluationResult evaluationResult = evaluationResultService.getById(evaluationResultId);
		return R.ok(evaluationResult);
		//return R.ok(evaluationResultService.getById(evaluationResultId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param evaluationResult (服务商/供应商评价结果)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_evaluation_result_get')" )
    public R<IPage<EvaluationResult>> getEvaluationResultPage(Page<EvaluationResult> page, EvaluationResult evaluationResult, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<EvaluationResult> evaluationResultSList = evaluationResultService.page(page, Wrappers.<EvaluationResult>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(evaluationResultSList);
//		return R.ok(evaluationResultService.page(page, Wrappers.query(evaluationResult)));
    }


    /**
     * 新增(服务商/供应商评价结果)
     * @param evaluationResult (服务商/供应商评价结果)
     * @return R
     */
    @ApiOperation(value = "新增(服务商/供应商评价结果)", notes = "新增(服务商/供应商评价结果)")
    @SysLog("新增(服务商/供应商评价结果)" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_evaluation_result_add')" )
    public R<EvaluationResult> postAddObject(@RequestBody EvaluationResult evaluationResult, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        evaluationResult.setEvaluationResultId(IdWorker.get32UUID().toUpperCase());
		if (evaluationResultService.save(evaluationResult)) {
			return R.ok(evaluationResult, "对象创建成功");
		}
		else {
			return R.failed(evaluationResult, "对象创建失败");
		}
    }

    /**
     * 修改(服务商/供应商评价结果)
     * @param evaluationResult (服务商/供应商评价结果)
     * @return R
     */
    @ApiOperation(value = "修改(服务商/供应商评价结果)", notes = "修改(服务商/供应商评价结果)")
    @SysLog("修改(服务商/供应商评价结果)" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_evaluation_result_edit')" )
    public R<EvaluationResult> putUpdateById(@RequestBody EvaluationResult evaluationResult, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (evaluationResultService.updateById(evaluationResult)) {
			return R.ok(evaluationResult, "保存对象成功");
		}
		else {
			return R.failed(evaluationResult, "保存对象失败");
		}
    }

    /**
     * 通过id删除(服务商/供应商评价结果)
     * @param evaluationResultId id
     * @return R
     */
    @ApiOperation(value = "通过id删除(服务商/供应商评价结果)", notes = "通过id删除(服务商/供应商评价结果)")
    @SysLog("通过id删除(服务商/供应商评价结果)" )
    @DeleteMapping("/{evaluationResultId}" )
    @PreAuthorize("@pms.hasPermission('reagent_evaluation_result_del')" )
    public R<EvaluationResult> deleteById(@PathVariable String evaluationResultId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        EvaluationResult oldEvaluationResult = evaluationResultService.getById(evaluationResultId);

		if (evaluationResultService.removeById(evaluationResultId)) {
			return R.ok(oldEvaluationResult, "对象删除成功");
		}
		else {
			return R.failed(oldEvaluationResult, "对象删除失败");
		}

    }

}
