package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.dto.AuditAndApproveDTO;
import digital.laboratory.platform.reagent.dto.PurchaseCatalogueDTO;
import digital.laboratory.platform.reagent.entity.CatalogueDetails;
import digital.laboratory.platform.reagent.entity.PurchaseCatalogue;
import digital.laboratory.platform.reagent.service.CatalogueDetailsService;
import digital.laboratory.platform.reagent.service.PurchaseCatalogueService;
import digital.laboratory.platform.reagent.vo.PurchaseCatalogueVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.security.Principal;
import java.util.List;

/**
 * (采购目录)
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe (采购目录) 前端控制器
 * <p>
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 * 为前端提供数据, 接受前端的数据
 * 为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 * 接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/purchase_catalogue")
@Api(value = "purchase_catalogue", tags = "采购目录管理")
public class PurchaseCatalogueController {

    @Autowired
    private final PurchaseCatalogueService purchaseCatalogueService;
    @Autowired
    private final CatalogueDetailsService catalogueDetailsService;

    /**
     * 通过id查询(采购目录)
     *
     * @param purchaseCatalogueId id
     * @return R
     */
    @ApiOperation(value = "通过id查询", notes = "通过id查询")
    @GetMapping("/{purchaseCatalogueId}")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_get')")
    public R<PurchaseCatalogue> getById(@PathVariable("purchaseCatalogueId") String purchaseCatalogueId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseCatalogueVO catalogueVo = purchaseCatalogueService.getPurchaseCatalogueVO(purchaseCatalogueId);

        if (catalogueVo != null) {
            return R.ok(catalogueVo);
        } else {
            return R.failed("未查询到该信息");
        }


    }

    /**
     * 分页查询
     *
     * @param page              分页对象
     * @param purchaseCatalogue (采购目录)
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_get')")
    public R<IPage<PurchaseCatalogueVO>> getPurchaseCataloguePage(Page<PurchaseCatalogue> page, PurchaseCatalogue purchaseCatalogue, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        IPage<PurchaseCatalogueVO> purchaseCatalogueVOPage = purchaseCatalogueService.getPurchaseCatalogueVOPage(page, Wrappers.<PurchaseCatalogue>query().eq("create_by", dlpUser.getId())
                .orderByDesc("create_time"));

        return R.ok(purchaseCatalogueVOPage);

    }


    /**
     * 新增(采购目录)
     *
     * @param purchaseCatalogueDTOList (采购目录)
     * @return R
     */
    @ApiOperation(value = "新增(采购目录)", notes = "新增(采购目录)")
    @SysLog("新增(采购目录)")
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_add')")
    public R<PurchaseCatalogue> postAddObject(@RequestBody List<PurchaseCatalogueDTO> purchaseCatalogueDTOList, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseCatalogue purchaseCatalogue = new PurchaseCatalogue();

        List<CatalogueDetails> catalogueDetailsList = purchaseCatalogueService.addCatalogue(dlpUser, purchaseCatalogueDTOList, purchaseCatalogue);

        if (purchaseCatalogueService.save(purchaseCatalogue) & catalogueDetailsService.saveBatch(catalogueDetailsList)) {
            return R.ok(purchaseCatalogue, "保存成功");
        } else {
            return R.failed(purchaseCatalogue, "保存失败");
        }
    }

    /**
     * 修改(采购目录)
     *
     * @param purchaseCatalogueDto (采购目录)
     * @return R
     */
    @ApiOperation(value = "修改采购目录明细", notes = "修改采购目录明细")
    @SysLog("修改明细采购目录明细")
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_edit')")
    public R<CatalogueDetails> putUpdateById(@RequestBody PurchaseCatalogueDTO purchaseCatalogueDto, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CatalogueDetails catalogueDetails = purchaseCatalogueService.editCatalogue(purchaseCatalogueDto);

        if (catalogueDetailsService.updateById(catalogueDetails)) {
            return R.ok(catalogueDetails, "修改成功");
        } else {
            return R.failed(catalogueDetails, "修改失败");
        }
    }

    /**
     * 通过id删除(采购目录)
     *
     * @param purchaseCatalogueId id
     * @return R
     */
    @ApiOperation(value = "通过id删除采购目录", notes = "通过id删除采购目录")
    @SysLog("通过id删除采购目录")
    @DeleteMapping("/{purchaseCatalogueId}")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_del')")
    public R<PurchaseCatalogue> deleteById(@PathVariable String purchaseCatalogueId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseCatalogue oldPurchaseCatalogue = purchaseCatalogueService.getById(purchaseCatalogueId);

        if (oldPurchaseCatalogue == null) {
            return R.failed("未能查到当前信息");
        }
        List<CatalogueDetails> list = purchaseCatalogueService.delCatalogue(purchaseCatalogueId);

        if (list!=null) {
            if (oldPurchaseCatalogue.getStatus()==0&catalogueDetailsService.removeBatchByIds(list)&purchaseCatalogueService.removeById(oldPurchaseCatalogue)){
                return R.ok(oldPurchaseCatalogue, "目录删除成功");
            }else return R.failed("目录删除失败");
        } else {
            if (oldPurchaseCatalogue.getStatus()==0&purchaseCatalogueService.removeById(oldPurchaseCatalogue)){
                return R.ok(oldPurchaseCatalogue, "目录删除成功");
            }else return R.failed("目录删除失败");
        }
    }

    /**
     * 通过id删除采购目录明细
     *
     * @param catalogueDetailsId
     * @return R
     */
    @ApiOperation(value = "通过id删除采购目录明细", notes = "通过id删除采购目录明细")
    @SysLog("通过id删除采购目录")
    @DeleteMapping("details/{catalogueDetailsId}")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_del')")
    public R<CatalogueDetails> deleteDetailsById(@PathVariable String catalogueDetailsId, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        CatalogueDetails byId = catalogueDetailsService.getById(catalogueDetailsId);

        if (catalogueDetailsService.removeById(byId)){
            return R.ok(byId,"删除明细成功");
        }else {
            return R.failed("删除失败");
        }
    }

    /**
     * 提交采购目录
     *
     * @param purchaseCatalogueId
     * @return R
     */
    @ApiOperation(value = "提交采购目录", notes = "提交采购目录")
    @SysLog("提交采购目录")
    @PutMapping("/commit/{purchaseCatalogueId}")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_edit')")
    public R<PurchaseCatalogue> commitById(@PathVariable String purchaseCatalogueId, HttpServletRequest theHttpServletRequest) {

        PurchaseCatalogue purchaseCatalogue = purchaseCatalogueService.commitById(purchaseCatalogueId);

        if (purchaseCatalogue!=null&purchaseCatalogueService.updateById(purchaseCatalogue)){
            return R.ok(purchaseCatalogue,"提交成功");
        }else {
            return R.failed("提交失败");
        }
    }

    /**
     * 审核采购目录
     *
     * @param auditAndApproveDto
     * @return R
     */
    @ApiOperation(value = "审核采购目录", notes = "审核采购目录")
    @SysLog("一级审核采购目录")
    @PutMapping("/primary/audit")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_edit')")
    public R<PurchaseCatalogue> primaryAuditCatalogue(@RequestBody AuditAndApproveDTO auditAndApproveDto, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseCatalogue purchaseCatalogue = purchaseCatalogueService.primaryAuditCatalogue(auditAndApproveDto,dlpUser);

        if (purchaseCatalogue!=null&purchaseCatalogueService.updateById(purchaseCatalogue)){
            return R.ok(purchaseCatalogue,"审核成功");
        }else {
            return R.failed("审核失败");
        }
    }

    /**
     * 审核采购目录
     *
     * @param auditAndApproveDto
     * @return R
     */
    @ApiOperation(value = "审核采购目录", notes = "审核采购目录")
    @SysLog("二级审核采购目录")
    @PutMapping("/secondary/audit")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_edit')")
    public R<PurchaseCatalogue> secondaryAuditCatalogue(@RequestBody AuditAndApproveDTO auditAndApproveDto, HttpServletRequest theHttpServletRequest) {

        Principal principal = theHttpServletRequest.getUserPrincipal();

        DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        PurchaseCatalogue purchaseCatalogue = purchaseCatalogueService.secondaryAuditCatalogue(auditAndApproveDto,dlpUser);

        if (purchaseCatalogue!=null&purchaseCatalogueService.updateById(purchaseCatalogue)){
            return R.ok(purchaseCatalogue,"审核成功");
        }else {
            return R.failed("审核失败");
        }
    }

    /**
     * 发布采购目录
     *
     * @param purchaseCatalogueId
     * @return R
     */
    @ApiOperation(value = "发布采购目录", notes = "发布采购目录")
    @SysLog("发布采购目录")
    @PutMapping("/release/{purchaseCatalogueId}")
    @PreAuthorize("@pms.hasPermission('reagent_purchase_catalogue_edit')")
    public R<PurchaseCatalogue> releaseById(@PathVariable String purchaseCatalogueId, HttpServletRequest theHttpServletRequest) {

        PurchaseCatalogue purchaseCatalogue = purchaseCatalogueService.releaseCatalogue(purchaseCatalogueId);

        if (purchaseCatalogue!=null&purchaseCatalogueService.updateById(purchaseCatalogue)){
            return R.ok(purchaseCatalogue,"发布成功");
        }else {
            return R.failed("发布失败");
        }
    }
    }


