package digital.laboratory.platform.reagent.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.IdWorker;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import digital.laboratory.platform.common.core.util.R;
import digital.laboratory.platform.common.log.annotation.SysLog;
import digital.laboratory.platform.common.mybatis.security.service.DLPUser;
import digital.laboratory.platform.reagent.entity.SignedBatchList;
import digital.laboratory.platform.reagent.service.SignedBatchListService;
import org.springframework.security.access.prepost.PreAuthorize;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.RequiredArgsConstructor;
import org.springframework.security.oauth2.provider.OAuth2Authentication;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.security.Principal;

/**
 * 签收批次明细
 *
 * @author Zhang Xiaolong created at  2023-03-10
 * @describe 签收批次明细 前端控制器
 *
 * 这是与表示层的接口, 不应该接业务逻辑写在这里, 业务逻辑应该写在 service 中
 * 这里写什么:
 *     为前端提供数据, 接受前端的数据
 *     为前端提供的数据, 从 service 取得后, 可以做一些适当的加工, 这种加工不是业务层面的, 只能是数据格式上, 为方便前端处理
 *     接受前端的数据, 每一个函数的参数可以先做一些整理后, 再调用 service 中的函数。这里对参数的整理, 应该只是格式上的, 而不能是业务上的
 * 数据层在 mapper 中, 数据层不涉及业务, 只管技术上的 对象<->表 之间的转换
 */
@RestController
@RequiredArgsConstructor
@RequestMapping("/signed_batch_list" )
@Api(value = "signed_batch_list", tags = "签收批次明细管理")
public class SignedBatchListController {

    private final  SignedBatchListService signedBatchListService;

	/**
	 * 通过id查询签收批次明细
	 * @param signedBatchListId id
	 * @return R
	 */
	@ApiOperation(value = "通过id查询", notes = "通过id查询")
	@GetMapping("/{signedBatchListId}" )
	@PreAuthorize("@pms.hasPermission('reagent_signed_batch_list_get')" )
	public R<SignedBatchList> getById(@PathVariable("signedBatchListId" ) String signedBatchListId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SignedBatchList signedBatchList = signedBatchListService.getById(signedBatchListId);
		return R.ok(signedBatchList);
		//return R.ok(signedBatchListService.getById(signedBatchListId));
	}

	/**
     * 分页查询
     * @param page 分页对象
     * @param signedBatchList 签收批次明细
     * @return
     */
    @ApiOperation(value = "分页查询", notes = "分页查询")
    @GetMapping("/page" )
    @PreAuthorize("@pms.hasPermission('reagent_signed_batch_list_get')" )
    public R<IPage<SignedBatchList>> getSignedBatchListPage(Page<SignedBatchList> page, SignedBatchList signedBatchList, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		IPage<SignedBatchList> signedBatchListSList = signedBatchListService.page(page, Wrappers.<SignedBatchList>query()
				.eq("create_by", dlpUser.getId())
				.orderByDesc("create_time")
		);
		return R.ok(signedBatchListSList);
//		return R.ok(signedBatchListService.page(page, Wrappers.query(signedBatchList)));
    }


    /**
     * 新增签收批次明细
     * @param signedBatchList 签收批次明细
     * @return R
     */
    @ApiOperation(value = "新增签收批次明细", notes = "新增签收批次明细")
    @SysLog("新增签收批次明细" )
    @PostMapping
    @PreAuthorize("@pms.hasPermission('reagent_signed_batch_list_add')" )
    public R<SignedBatchList> postAddObject(@RequestBody SignedBatchList signedBatchList, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        signedBatchList.setSignedBatchListId(IdWorker.get32UUID().toUpperCase());
		if (signedBatchListService.save(signedBatchList)) {
			return R.ok(signedBatchList, "对象创建成功");
		}
		else {
			return R.failed(signedBatchList, "对象创建失败");
		}
    }

    /**
     * 修改签收批次明细
     * @param signedBatchList 签收批次明细
     * @return R
     */
    @ApiOperation(value = "修改签收批次明细", notes = "修改签收批次明细")
    @SysLog("修改签收批次明细" )
    @PutMapping
    @PreAuthorize("@pms.hasPermission('reagent_signed_batch_list_edit')" )
    public R<SignedBatchList> putUpdateById(@RequestBody SignedBatchList signedBatchList, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

		if (signedBatchListService.updateById(signedBatchList)) {
			return R.ok(signedBatchList, "保存对象成功");
		}
		else {
			return R.failed(signedBatchList, "保存对象失败");
		}
    }

    /**
     * 通过id删除签收批次明细
     * @param signedBatchListId id
     * @return R
     */
    @ApiOperation(value = "通过id删除签收批次明细", notes = "通过id删除签收批次明细")
    @SysLog("通过id删除签收批次明细" )
    @DeleteMapping("/{signedBatchListId}" )
    @PreAuthorize("@pms.hasPermission('reagent_signed_batch_list_del')" )
    public R<SignedBatchList> deleteById(@PathVariable String signedBatchListId, HttpServletRequest theHttpServletRequest) {
		Principal principal = theHttpServletRequest.getUserPrincipal();
		DLPUser dlpUser = (DLPUser) ((OAuth2Authentication) principal).getUserAuthentication().getPrincipal();

        SignedBatchList oldSignedBatchList = signedBatchListService.getById(signedBatchListId);

		if (signedBatchListService.removeById(signedBatchListId)) {
			return R.ok(oldSignedBatchList, "对象删除成功");
		}
		else {
			return R.failed(oldSignedBatchList, "对象删除失败");
		}

    }

}
